package org.bench4q.agent.datacollector.impl;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.bench4q.agent.Main;
import org.bench4q.agent.datacollector.DataCollector;
import org.bench4q.agent.scenario.engine.BehaviorResult;
import org.bench4q.agent.scenario.engine.PageResult;
import org.bench4q.agent.storage.StorageHelper;
import org.bench4q.share.models.agent.BehaviorResultModel;

public class DataCollectorImpl implements DataCollector {
	protected StorageHelper storageHelper;
	private final UUID testId;
	private final PagesResultCollector pagesResultCollector;
	private final ScenarioResultCollector scenarioResultCollector;
	private final BehaviorsResultCollector behaviorsResultCollector;

	protected StorageHelper getStorageHelper() {
		return storageHelper;
	}

	public void setStorageHelper(StorageHelper storageHelper) {
		this.storageHelper = storageHelper;
	}

	public DataCollectorImpl(UUID testId, StorageHelper storageHelper) {
		this.storageHelper = storageHelper;
		this.testId = testId;
		this.pagesResultCollector = new PagesResultCollector();
		this.behaviorsResultCollector = new BehaviorsResultCollector();
		this.scenarioResultCollector = new ScenarioResultCollector();
	}

	public void add(final BehaviorResult behaviorResult) {
		if (behaviorResult == null) {
			return;
		}
		saveItAsync(behaviorResult);
		this.scenarioResultCollector.add(behaviorResult);
		this.behaviorsResultCollector.add(behaviorResult);
	}

	private void saveItAsync(final BehaviorResult behaviorResult) {
		if (!Main.IS_TO_SAVE_DETAIL) {
			return;
		}
		Runnable runnable = new Runnable() {
			public void run() {
				storageHelper.getLocalStorage().writeFile(
						buildBehaviorResultModel(behaviorResult)
								.getModelString(),
						calculateSavePath(behaviorResult));
			}
		};
		ExecutorService executorService = Executors
				.newSingleThreadScheduledExecutor();
		executorService.execute(runnable);
		executorService.shutdown();
	}

	private BehaviorResultModel buildBehaviorResultModel(
			BehaviorResult behaviorResult) {
		BehaviorResultModel resultModel = new BehaviorResultModel();
		resultModel.setBehaviorId(behaviorResult.getBehaviorId());
		resultModel.setBehaviorName(behaviorResult.getBehaviorName());
		resultModel.setContentLength(behaviorResult.getContentLength());
		resultModel.setContentType(behaviorResult.getContentType());
		resultModel.setEndDate(behaviorResult.getEndDate());
		resultModel.setPluginId(behaviorResult.getPluginId());
		resultModel.setPluginName(behaviorResult.getPluginName());
		resultModel.setResponseTime(behaviorResult.getResponseTime());
		resultModel.setShouldBeCountResponseTime(behaviorResult
				.isShouldBeCountResponseTime());
		resultModel.setStartDate(behaviorResult.getStartDate());
		resultModel.setStatusCode(behaviorResult.getStatusCode());
		resultModel.setSuccessCount(behaviorResult.getSuccessCount());
		resultModel.setFailCount(behaviorResult.getFailCount());
		return resultModel;
	}

	public void add(PageResult pageResult) {
		this.pagesResultCollector.add(pageResult);
	}

	public Object getPageBriefStatistics(int pageId) {
		return this.pagesResultCollector.getPageBriefStatistics(pageId);
	}

	protected String calculateSavePath(BehaviorResult behaviorResult) {
		Date now = new Date();
		return "DetailResults" + System.getProperty("file.separator")
				+ new SimpleDateFormat("yyyyMMdd").format(now)
				+ System.getProperty("file.separator") + this.testId + "_"
				+ behaviorResult.getBehaviorId() + "_"
				+ new SimpleDateFormat("hhmm") + ".txt";
	}

	public Object getScenarioBriefStatistics() {
		return this.scenarioResultCollector.getScenarioBriefStatistics();
	}

	public Object getBehaviorBriefStatistics(int id) {
		return this.behaviorsResultCollector.getStatistics(id);
	}

}
