package org.bench4q.agent.datacollector.impl;

import org.bench4q.agent.scenario.engine.BehaviorResult;
import org.bench4q.share.models.agent.statistics.AgentBriefStatusModel;

/**
 * This class collect the behavior result and statistic it.
 * 
 * @author coderfengyun
 * 
 */
public class ScenarioResultCollector {
	private long timeOfPreviousCall;
	private long failCountOfThisCall;
	private long successCountOfThisCall;
	private long totalResponseTimeOfThisCall;
	private long maxResponseTimeOfThisCall;
	private long minResponseTimeOfThisCall;
	private long totalSqureResponseTimeOfThisCall;
	private long cumulativeSucessfulCount;
	private long cumulativeFailCount;
	private static long TIME_UNIT = 1000;

	private void setTimeOfPreviousCall(long timeOfPreviousCall) {
		this.timeOfPreviousCall = timeOfPreviousCall;
	}

	private void setFailCountOfThisCall(long failCountOfThisCall) {
		this.failCountOfThisCall = failCountOfThisCall;
	}

	private long getSuccessCountOfThisCall() {
		return successCountOfThisCall;
	}

	private void setSuccessCountOfThisCall(long successCountOfThisCall) {
		this.successCountOfThisCall = successCountOfThisCall;
	}

	private void setTotalResponseTimeOfThisCall(long totalResponseTimeOfThisCall) {
		this.totalResponseTimeOfThisCall = totalResponseTimeOfThisCall;
	}

	private long getMaxResponseTimeOfThisCall() {
		return maxResponseTimeOfThisCall;
	}

	private void setMaxResponseTimeOfThisCall(long maxResponseTimeOfThisCall) {
		this.maxResponseTimeOfThisCall = maxResponseTimeOfThisCall;
	}

	private long getMinResponseTimeOfThisCall() {
		return minResponseTimeOfThisCall;
	}

	private void setMinResponseTimeOfThisCall(long minResponseTimeOfThisCall) {
		this.minResponseTimeOfThisCall = minResponseTimeOfThisCall;
	}

	private void setTotalSqureResponseTimeOfThisCall(
			long totalSqureResponseTimeOfThisCall) {
		this.totalSqureResponseTimeOfThisCall = totalSqureResponseTimeOfThisCall;
	}

	private void setCumulativeSucessfulCount(long cumulativeSucessfulCount) {
		this.cumulativeSucessfulCount = cumulativeSucessfulCount;
	}

	private void setCumulativeFailCount(long cumulativeFailCount) {
		this.cumulativeFailCount = cumulativeFailCount;
	}

	public ScenarioResultCollector() {
		init();
	}

	private void init() {
		reset();
		this.setCumulativeFailCount(0);
		this.setCumulativeSucessfulCount(0);
	}

	private void reset() {
		this.setTimeOfPreviousCall(System.currentTimeMillis());
		this.setFailCountOfThisCall(0);
		this.setMaxResponseTimeOfThisCall(Long.MIN_VALUE);
		this.setMinResponseTimeOfThisCall(Long.MAX_VALUE);
		this.setSuccessCountOfThisCall(0);
		this.setTotalResponseTimeOfThisCall(0);
		this.setTotalSqureResponseTimeOfThisCall(0);
	}

	// ///////////////////////////////
	// DataStatistics Interface start
	// ///////////////////////////////

	public void add(BehaviorResult behaviorResult) {
		this.successCountOfThisCall += behaviorResult.getSuccessCount();
		this.failCountOfThisCall += behaviorResult.getFailCount();
		if (behaviorResult.getSuccessCount() == 0)
			return;
		this.totalResponseTimeOfThisCall += behaviorResult.getResponseTime();
		this.totalSqureResponseTimeOfThisCall += Math.pow(
				behaviorResult.getResponseTime(), 2);
		if (behaviorResult.getResponseTime() > this
				.getMaxResponseTimeOfThisCall()) {
			this.setMaxResponseTimeOfThisCall(behaviorResult.getResponseTime());
		}
		if (behaviorResult.getResponseTime() < this
				.getMinResponseTimeOfThisCall()) {
			this.setMinResponseTimeOfThisCall(behaviorResult.getResponseTime());
		}
	}

	public AgentBriefStatusModel getScenarioBriefStatistics() {
		AgentBriefStatusModel result = new AgentBriefStatusModel();
		result.setTimeFrame(System.currentTimeMillis()
				- this.timeOfPreviousCall);
		if (this.getSuccessCountOfThisCall() == 0) {
			result.setMaxResponseTime(0);
			result.setMinResponseTime(0);
		} else {
			result.setMinResponseTime(this.minResponseTimeOfThisCall);
			result.setMaxResponseTime(this.maxResponseTimeOfThisCall);
		}
		this.cumulativeSucessfulCount += this.successCountOfThisCall;
		result.setSuccessCountFromBegin(this.cumulativeSucessfulCount);
		this.cumulativeFailCount += this.failCountOfThisCall;
		result.setFailCountFromBegin(this.cumulativeFailCount);
		if (result.getTimeFrame() == 0) {
			result.setSuccessThroughputThisTime(0);
			result.setFailThroughputThisTime(0);
		} else {
			result.setSuccessThroughputThisTime(this.successCountOfThisCall
					* TIME_UNIT / result.getTimeFrame());
			result.setFailThroughputThisTime(this.failCountOfThisCall
					* TIME_UNIT / result.getTimeFrame());
		}
		result.setTotalResponseTimeThisTime(this.totalResponseTimeOfThisCall);
		result.setSuccessCountThisTime(this.successCountOfThisCall);
		result.setFailCountThisTime(this.failCountOfThisCall);
		result.setTotalSqureResponseTimeThisTime(this.totalSqureResponseTimeOfThisCall);
		reset();
		return result;
	}

	// ///////////////////////////////
	// DataStatistics Interface end
	// ///////////////////////////////
	/**
	 * For the failed one, only the fail count statistics will be added, Others
	 * of this failed one will be ignored.
	 * 
	 * @param behaviorResult
	 */

}
