package org.bench4q.agent.scenario.node;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.bench4q.agent.scenario.instruction.GotoInstruction;
import org.bench4q.agent.scenario.instruction.Instruction;
import org.bench4q.agent.scenario.instruction.TestInstruction;
import org.bench4q.share.models.agent.scriptrecord.BehaviorModel;
import org.bench4q.share.models.agent.scriptrecord.IfBehaviorModel;

public class IfBehavior extends ConditionedBehavior {
	private List<Behavior> thenBehaviors;
	private List<Behavior> elseBehaviors;

	public IfBehavior(IfBehaviorModel model) {
		this.thenBehaviors = new ArrayList<Behavior>();
		this.elseBehaviors = new ArrayList<Behavior>();
		this.condition = new ConditionBehavior(model.getCondition());
		for (BehaviorModel then : model.getThenBehaviors()) {
			this.thenBehaviors.add(Behavior.buildWith(then));
		}

		for (BehaviorModel elseModel : model.getElseBehaviors()) {
			this.elseBehaviors.add(Behavior.buildWith(elseModel));
		}
	}

	/**
	 * both code and conditions are result
	 */
	@Override
	public void compile(ArrayList<Instruction> code,
			Stack<TestInstruction> conditions) {
		TestInstruction testInstruction;
		GotoInstruction gotoInstruction;
		condition.checkCondition(code, conditions);

		code.add(testInstruction = new TestInstruction(condition.getUse(),
				condition.getName(), condition.getParameters()));

		for (Behavior behavior : thenBehaviors) {
			behavior.compile(code, conditions);
		}
		if (this.thenBehaviors.size() != 0) {
			code.add(gotoInstruction = new GotoInstruction(0));
			testInstruction.setFalseLabel(code.size());
			for (Behavior behavior : this.elseBehaviors) {
				behavior.compile(code, conditions);
			}
			gotoInstruction.label = code.size();
		} else {
			testInstruction.setFalseLabel(code.size());
		}
	}
}
